package com.nisum.application.controller;

import com.nisum.application.dto.ProfileDTO;
import com.nisum.application.model.Profile;
import com.nisum.application.service.ProfileService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import javax.validation.Valid;
import javax.validation.constraints.Pattern;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;

@RestController
@RequestMapping("/profile")
@Slf4j
public class ProfileController {

    @Autowired
    private ProfileService profileService;

    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public Mono<Profile> createProfile(@Valid @RequestBody ProfileDTO profileDTO) {
        Profile profile = convertToEntity(profileDTO);

        return profileService.createProfile(profile)
                .doOnSuccess(savedProfile -> log.info("Profile created successfully: {}", savedProfile))
                .doOnError(error -> {
                    log.error("Error creating profile", error);
                    if (error instanceof ResponseStatusException) {
                        ResponseStatusException exception = (ResponseStatusException) error;
                        log.error("HTTP Status: {}, Message: {}", exception.getStatus(), exception.getReason());
                    }
                });
    }



    private Profile convertToEntity(ProfileDTO profileDTO) {
        Profile profile = new Profile();
        profile.setFirstName(profileDTO.getFirstName());
        profile.setLastName(profileDTO.getLastName());
        profile.setDateOfBirth(LocalDate.parse(profileDTO.getDob(), DateTimeFormatter.ofPattern("MM-dd-yyyy")));
        profile.setPermanentAccountNumber(profileDTO.getPan());
        profile.setAdharNumber(profileDTO.getAdhar());
        profile.setMobileNumber(profileDTO.getMobile());
        return profile;
    }


    @GetMapping
    public Flux<Profile> searchProfiles(
            @RequestParam(required = false) String firstName,
            @RequestParam(required = false) String lastName,
            @RequestParam(required = false) @Pattern(regexp = "^[A-Z]{5}[0-9]{4}[A-Z]$", message = "Invalid PAN format") String pan,
            @RequestParam(required = false) @Pattern(regexp = "^[1-9][0-9]{11}$", message = "Invalid Aadhar format") String adhar,
            @RequestParam(required = false) @Pattern(regexp = "^[6-9][0-9]{9}$", message = "Invalid mobile format") String mobile,
            @RequestParam(required = false) String dob) {

        LocalDate dateOfBirth = null;
        if (dob != null) {
            try {
                dateOfBirth = LocalDate.parse(dob, DateTimeFormatter.ofPattern("MM-dd-yyyy")); // Ensure correct date format
            } catch (DateTimeParseException e) {
                return Flux.error(new IllegalArgumentException("Invalid date format. Expected format: MM-DD-YYYY"));
            }
        }

        log.info("Received request to search profiles with criteria - FirstName: {}, LastName: {}, PAN: {}, Aadhar: {}, Mobile: {}, DOB: {}",
                firstName, lastName, pan, adhar, mobile, dob);
        return profileService.searchProfiles(firstName, lastName, pan, adhar, mobile, dateOfBirth)
                .doOnError(error -> log.error("Error searching profiles", error));
    }

    @PutMapping
    @ResponseStatus(HttpStatus.OK)
    public Mono<Profile> updateProfile(@Valid @RequestBody ProfileDTO profileUpdateDTO) {
        return profileService.updateProfile(profileUpdateDTO)
                .doOnSuccess(updatedProfile -> log.info("Profile updated successfully: {}", updatedProfile))
                .doOnError(error -> log.error("Error updating profile", error));
    }

    @DeleteMapping("/{profileId}")
    public Mono<ResponseEntity<Object>> deleteProfile(@PathVariable String profileId) {
        return profileService.deleteProfileById(profileId)
                .map(deleted -> ResponseEntity.noContent().build())
                .doOnError(error -> log.error("Error deleting profile", error));
    }
}




